@@ -2,20 +2,32 @@ from inspect import isclass
 from abc import ABCMeta, abstractmethod
 
 
-class StateMachineCrawlerError(Exception):
-    """ Error to be raised when:
+class StateMachineError(Exception):
+    """ Base error to be raise by the toolkit """
 
-    - there are issues with initialization of the state state machine
-    - it is impossible to perform state transition because the state is unreachable
+
+class TransitionError(StateMachineError):
+    """ Raised if the transition could not be performed.
+
+    Failure could happen because:
+
+    - target state is not reachable
+    - state verification failed
+
+    NOTE: if transition itself fails (i.e. exception in the *move* method) - the exception is raised as is
     """
 
 
+class DeclarationError(StateMachineError):
+    """ Raised if something is wrong with the state machine declaration in general """
+
+
 class Transition(object):
     """ Represents a transformation of the system from one state into another
 
-    Transitions have *_system* attribute that represents the entity with which all the states are associated.
+    Transitions have a *_system* attribute that represents the entity with which transitions' states are associated.
 
-    Class definition of the transition must have:
+    Class definitions of the transitions must have:
 
     cost (int)
         Relative *price* of the transition. Transitions that take longer time to run are more *expensive*. The *cost*
@@ -28,7 +40,7 @@ class Transition(object):
     The only difference between *target_state* and *source_state* is a direction of the relationship.
 
     Note: there can be only *target_state* or only *source_state* because if a transition from state **A** to state
-    **B** is possible it does not imply that the opposite transition is possible.
+    **B** is possible it does not at all imply that the opposite transition can be performed the same way.
     """
     __metaclass__ = ABCMeta
     cost = 1
@@ -40,7 +52,7 @@ class Transition(object):
     @abstractmethod
     def move(self):
         """
-        Performs the actions to move from state one to state two.
+        Performs the actions to move from one state to another.
         """
 
     @classmethod
@@ -49,7 +61,7 @@ class Transition(object):
         Links an existing transition with a specific state.
 
         This method exists to avoid creating unnecessary subclasses in the situation when multiple states can perform
-        the similar transitions.
+        similar transitions.
         """
         tstate = target_state
         sstate = source_state
@@ -60,7 +72,7 @@ class Transition(object):
         return NewTransition
 
 
-class StateMetaClass(type):
+class StateMetaClass(ABCMeta):
 
     def __init__(self, name, bases, attrs):
         super(StateMetaClass, self).__init__(name, bases, attrs)
@@ -70,6 +82,7 @@ class StateMetaClass(type):
             if not (isclass(attr) and issubclass(attr, Transition)):
                 continue
             if attr.target_state:
+                attr.source_state = self
                 if attr.target_state == "self":
                     attr.target_state = self
                     self.transition_map[self] = attr
@@ -78,56 +91,25 @@ class StateMetaClass(type):
             elif attr.source_state:
                 class RelatedTransition(attr):
                     target_state = self
-                    source_state = None
+                RelatedTransition.__name__ = name
                 attr.source_state.transition_map[self] = RelatedTransition
             else:
-                raise StateMachineCrawlerError("No target nor source state is defined for %r" % attr)
+                raise DeclarationError("No target nor source state is defined for %r" % attr)
 
 
 class State(object):
-    """ A base class for any state of the system """
+    """ A base class for any state of the system
+
+    States have a *_system* attribute that represents the entity with which they are associated.
+    """
     __metaclass__ = StateMetaClass
 
-
-class InitialState(State):
-    """ Represents the initial state of the system. The initial state is reachable from ANY other state. """
-
-
-class InitialTransition(Transition):
-    """ A special transformation that configures the system to a blank state.
-
-    It must be possible to perform the transition at any point. I.e. all the states should be transitionable to
-    the initial state. It might me the most expensive transition in the system.
-
-    The transition is bound with :class:`InitialState <state_machine_crawler.InitialState>`. This should not be changed.
-    """
-    target_state = InitialState
+    def __init__(self, system):
+        self._system = system
 
     @abstractmethod
-    def move(self):
-        """ Any code responsible for configuring the system should be placed here """
-
-
-class ErrorState(State):
-    """ Represents a state of the system with abnormal conditions """
-
-
-class ErrorTransition(Transition):
-    """ Base class for managing a transition of the system into the ErrorState.
-
-    Exception is available as *_error* instance attribute.
-
-    The transition is bound with :class:`ErrorState <state_machine_crawler.ErrorState>`. This should not be changed.
-    """
-    target_state = ErrorState
-
-    def __init__(self, system, error):
-        super(ErrorTransition, self).__init__(system)
-        self._error = error
-
-    @abstractmethod
-    def move(self):
-        """ All error logging and/or error handling should be implemented here """
+    def verify(self):
+        """ Checks if the system ended up in a desired state. Should return a boolean. """
 
 
 def _get_cost(states):
@@ -140,7 +122,7 @@ def _get_cost(states):
     return cost
 
 
-def _find_shortest_path(graph, start, end, path=[]):
+def _find_shortest_path(graph, start, end, path=[], get_cost=_get_cost):
     """ Derived from `here <https://www.python.org/doc/essays/graphs/>`_
 
     Finds the shortest path between two states. Estimations are based on a sum of costs of all transitions.
@@ -153,9 +135,9 @@ def _find_shortest_path(graph, start, end, path=[]):
     shortest = None
     for node in graph[start]:
         if node not in path:
-            newpath = _find_shortest_path(graph, node, end, path)
+            newpath = _find_shortest_path(graph, node, end, path, get_cost)
             if newpath:
-                if not shortest or _get_cost(newpath) < _get_cost(shortest):
+                if not shortest or get_cost(newpath) < get_cost(shortest):
                     shortest = newpath
     return shortest
 
@@ -176,31 +158,53 @@ class StateMachineCrawler(object):
     """ The crawler is responsible for orchestrating the transitions of system's states
 
     system
-        All transition shall change its state.
+        All transitions shall change the internal state of this object.
     initial_transition (subclass of :class:`InitialTransition <state_machine_crawler.InitialTransition>`)
-        The first transition to be executed
-    error_transition (subclass of :class:`ErrorTransition <state_machine_crawler.ErrorTransition>`)
-        The transition to be executed when exceptional situation takes place
+        The first transition to be executed to move to the initial state
 
-    >>> scm = StateMachineCrawler(system_object, CustomIntialTransition, CustomErrorTransition)
+    >>> scm = StateMachineCrawler(system_object, CustomIntialTransition)
+
+    or
+
+    >>> scm = StateMachineCrawler.create(system_object, CustomIntialTransition)
     """
 
-    def __init__(self, system, initial_transition, error_transition):
+    @classmethod
+    def create(cls, system, initial_transition):
+        """ Instanciates and returns the crawler with a state that mostly resembles the one the *_system* is in.
+        The state does not necessarily have to be the initial one."""
+        instance = cls(system, initial_transition)
+        longest_distance = 0
+        current_state = None
+        for state in instance._state_graph:
+            if not state(system).verify():
+                continue
+            shortest_path = _find_shortest_path(instance._state_graph, initial_transition.target_state, state,
+                                                get_cost=len)
+            distance = len(shortest_path)
+            if distance > longest_distance:
+                longest_distance = distance
+                current_state = state
+            elif distance < longest_distance:
+                pass
+            else:
+                raise TransitionError("States %r and %r satisfy system's condition" % (current_state, state))
+        instance._current_state = current_state
+        return instance
+
+    def __init__(self, system, initial_transition):
         self._system = system
-        self._current_state = None
-        if not (isclass(initial_transition) and issubclass(initial_transition, InitialTransition)):
-            raise StateMachineCrawlerError("initial_transition must be InitialTransition subclass")
-        if not (isclass(error_transition) and issubclass(error_transition, ErrorTransition)):
-            raise StateMachineCrawlerError("error_transition must be ErrorTransition subclass")
+        self._current_state = self._current_transition = None
+        if not (isclass(initial_transition) and issubclass(initial_transition, Transition)):
+            raise DeclarationError("initial_transition must be a Transition subclass")
         self._initial_transition = initial_transition
         self._initial_state = initial_transition.target_state
-        self._error_transition = error_transition
         self._state_graph = self._init_state_graph()
 
     def _init_state_graph(self):
         initial_state = self._initial_transition.target_state
         if initial_state is None:
-            raise StateMachineCrawlerError("Initial transition has no target state")
+            raise DeclarationError("Initial transition has no target state")
         state_graph = _create_transition_map(initial_state)
         for source_state, target_states in state_graph.iteritems():
             target_states.add(initial_state)
@@ -215,7 +219,7 @@ class StateMachineCrawler(object):
         return self._current_state
 
     def move(self, state):
-        """ Performs a transition from the current state to the state passed as the argument
+        """ Performs a transition from the current state to the state passed as an argument
 
         state (subclass of :class:`State <state_machine_crawler.State>`)
             target state of the system
@@ -226,22 +230,22 @@ class StateMachineCrawler(object):
         """
         if state is self._initial_state or self._current_state is None:
             self._initial_transition(self._system).move()
+            if not self._initial_state(self._system).verify():
+                raise TransitionError("Getting to the initial state has failed")
             self._current_state = self._initial_state
             if state is self._initial_state:
                 return
         shortest_path = _find_shortest_path(self._state_graph, self._current_state, state)
         if shortest_path is None:
-            raise StateMachineCrawlerError("There is no way to achieve state %r" % state)
+            raise TransitionError("There is no way to achieve state %r" % state)
         if state is self._current_state:
             next_states = [state]
         else:
             next_states = shortest_path[1:]
         for next_state in next_states:
-            transition = self._current_state.transition_map[next_state]
-            try:
-                transition(self._system).move()
+            self._current_transition = transition = self._current_state.transition_map[next_state]
+            transition(self._system).move()
+            if next_state(self._system).verify():
                 self._current_state = next_state
-            except Exception, error:
-                self._error_transition(self._system, error).move()
-                self.move(self._initial_state)
-                break
+            else:
+                raise TransitionError("Move from state %r to state %r has failed" % (self._current_state, next_state))
